.\" author: Pierre Cassat
.TH  "markdown-extended-api" "7" "27-12-2014" "Version 0.1-gamma4" "PHP-Markdown-Extended Developer Manual"
.SH NAME
.PP
PHP-Markdown-Extended-API - Developer documentation of the internal API of PHP-Markdown-Extended.
.PP
The whole API is stored in the `\fS\\MarkdownExtended\\API\fP` namespace and is a set of interfaces
you must implement to use or override the parser or some of its objects.
.SH LIFE-CYCLE
.PP
The full schema of a Markdown parser usage could be:
.RS

.EX
        [source file content]   [options]    
.br
               ||                  ||
.br
               \\/                  \\/
.br
        ---------------        ----------                            -------------------
.br
        |  MD SOURCE  |   =>   | PARSER |   =>  [output format]  =>  | FORMATTED RESULT |
.br
        ---------------        ----------                            -------------------
.br
               /\\                  /\\                                         ||
.br
               ||                  ||                                         \\/
.br
            [string]        [ configuration ]                           [special infos]
.EE
.RE
.PP
The original Markdown source can be either a buffered string (a form field for example)
or the content of a Markdown file
.PP
We want to parse the source content with a hand on options used during this parsing
(no need to parse metadata in a content that will never have some for example)
.PP
Finally, we want to get a formatted content and to be able to retrieve certain infos
from it, such as its metadata, its menu or the footnotes of the whole parsed result
.PP
Additionally, it would be best that we can obtain a full formatted result simply but
can also pass this result through a template builder to construct a complex final string
.SH CODING RESULTS
.PP
The first item of this chain is assumed by the `\fS\\MarkdownExtended\\Content\fP` object.
It is a simple class that just stores different infos about a parsed content, such as its 
original source, the body of the result (the real parsed content), its menu, its metadata, 
its DOM ids and its footnotes. This object is overloadable but MUST implement the
`\fS\\MarkdownExtended\\API\\ContentInterface\fP` interface.
.PP
The second step is handled by the `\fS\\MarkdownExtended\\Parser\fP` object where lives the central
work of the syntax rules transformations. It depends on a configuration that can be reset
at every call. This object is overloadable but MUST implement the
`\fS\\MarkdownExtended\\API\\ParserInterface\fP` interface.
.PP
Finally, the whole thing is contained in the `\fS\\MarkdownExtended\\MarkdownExtended\fP` object
that is a kind of global container for the Markdown work.
.PP
All API classes finally used to create objects are defined as configuration entries like:
.RS

.EX
// the default API objects
.br
\'content_class'             => '\\MarkdownExtended\\Content',
.br
\'content_collection_class'  => '\\MarkdownExtended\\ContentCollection',
.br
\'parser_class'              => '\\MarkdownExtended\\Parser',
.br
\'templater_class'           => '\\MarkdownExtended\\Templater',
.br
\'grammar\\filter_class'      => '\\MarkdownExtended\\Grammar\\Filter',
.br
\'grammar\\tool_class'        => '\\MarkdownExtended\\Grammar\\Tool',
.br

.EE
.RE
.PP
Please see the `\fS\\MarkdownExtended\\Config\fP` object source for a full and up-to-date list.
.SH FULL USAGE
.SS The "kernel" object
.PP
Creation of the container as a singleton instance:
.RS

.EX
$mde = \\MarkdownExtended\\MarkdownExtended::create( options );
.br
// to retrieve the same instance after creation:
.br
$mde = \\MarkdownExtended\\MarkdownExtended::getInstance();
.br

.EE
.RE
.SS The \fIContent\fP object
.PP
Creation of a new content object:
.RS

.EX
// with a string:
.br
$source = new \\MarkdownExtended\\Content( $string );
.br

.br
// with a file to get content from:
.br
$source = new \\MarkdownExtended\\Content( null, $filepath );
.br

.EE
.RE
.PP
If you configured your own object, use:
.RS

.EX
// get or create your own object instance
.br
$source = \\MarkdownExtended\\MarkdownExtended::get(
.br
    'content', $config_as_an_array
.br
);
.br

.EE
.RE
.SS The \fIParser\fP object
.PP
Get the parser instance from the container:
.RS

.EX
$parser = $mde->get('Parser', $parser_options);    
.br

.EE
.RE
.PP
If you configured your own object, use:
.RS

.EX
// get or create your own object instance
.br
$source = \\MarkdownExtended\\MarkdownExtended::get(
.br
    'Parser', $parser_options_as_an_array
.br
);
.br

.EE
.RE
.SS The markdown process
.PP
Make the source transformation:
.RS

.EX
// this will return the Container
.br
$markdown = $parser->parse($source)
.br
    // and this will return the Content object transformed
.br
    ->getContent();
.br

.EE
.RE
.SS The transformed content
.PP
Then, get the transformed content and other infos from the \fIContent\fP object:
.RS

.EX
echo "<html><head>"
.br
    .$markdown->getMetadataHtml()   // the content metadata HTML formatted
.br
    ."</head><body>"
.br
    .$markdown->getBody()           // the content HTML body
.br
    ."<hr />"
.br
    .$markdown->getNotesHtml()      // the content footnotes HTML formatted
.br
    ."</body></html>";
.br

.EE
.RE
.PP
In case of a simple source (such as a textarea field):
.RS

.EX
echo $markdown->getBody();
.br

.EE
.RE
.PP
For simplest calls, a \fIHelper\fP is designed to allow usage of:
.RS

.EX
echo \\MarkdownExtended\\MarkdownExtended::getFullContent();
.br

.EE
.RE
.PP
that will return the exact same string as the one constructed above (a full HTML page
by default).
.SH COMPONENTS
.PP
The Internal classes (required and not overloadable) are:
.RS
.IP \(bu 
the API: `\fS\\MarkdownExtended\\API\fP`
.IP \(bu 
the "kernel" object: `\fS\\MarkdownExtended\\MarkdownExtended\fP`
.IP \(bu 
the configuration handler: `\fS\\MarkdownExtended\\Config\fP`
.IP \(bu 
the registry (works as a container): `\fS\\MarkdownExtended\\Registry\fP`
.IP \(bu 
the "output formatter" which depends on your chosen format: `\fS\\MarkdownExtended\\OutputFormatBag\fP`
.RE
.PP
The API classes, overloadables, are:
.RS
.IP \(bu 
the "parser" who will handle all parsing steps: `\fS\\MarkdownExtended\\Parser\fP`,
which must implement the `\fS\\MarkdownExtended\\API\\ParserInterface\fP`
.IP \(bu 
a "content" single object: `\fS\\MarkdownExtended\\Content\fP`,
which must implement the `\fS\\MarkdownExtended\\API\\ContentInterface\fP`
.IP \(bu 
a collection of "contents": `\fS\\MarkdownExtended\\ContentCollection\fP`,
which must implement the `\fS\\MarkdownExtended\\API\\CollectionInterface\fP`
.IP \(bu 
a "templater" object to load a parsed content in a template file: `\fS\\MarkdownExtended\\Templater\fP`,
which must implement the `\fS\\MarkdownExtended\\API\\TemplaterInterface\fP`
.RE
.PP
Each object is loaded as a service in the kernel and can be retrieved from the kernel instance
with a simple getter:
.RS

.EX
$object = \\MardownExtended\\MarkdownExtended::getInstance()->get( name );
.br

.EE
.RE
.PP
Trying to get it, if the object does not exist yet, it will be created.
.SH API KERNEL
.PP
The `\fS\\MarkdownExtended\\API\fP` is the central class object. It handles all the parsing
logic and acts like a services container for other API's objects.
.PP
The `\fS\\MarkdownExtended\\MarkdownExtended\fP` is the base public class object. It proposes
a large set of aliases to manage your contents (the original ones and their parsed results).
.RS

.EX
// creation of the singleton instance of \\MarkdownExtended\\MarkdownExtended
.br
$parser = \\MarkdownExtended\\MarkdownExtended::create( [options] );
.br

.EE
.RE
.PP
The best practice is to use the kernel as a singleton instance but you are allowed to use
it as a "classic" object creating it like any other PHP object.
.PP
The \fIMarkdownExtended\fP package can be simply call writing:
.RS

.EX
// creation of the singleton instance of \\MarkdownExtended\\MarkdownExtended
.br
$content = \\MarkdownExtended\\MarkdownExtended::create()
.br
    // get the \\MarkdownExtended\\Parser object passing it some options (optional)
.br
    ->get('Parser', $options)
.br
    // launch the transformation of a source content
.br
    ->parse( new \\MarkdownExtended\\Content($source) )
.br
    // get the result content object
.br
    ->getContent();
.br

.EE
.RE
.PP
This will load in \fI$content\fP the parsed HTML version of your original Markdown \fI$source\fP.
To get the part you need from the content, write:
.RS

.EX
echo $content->getBody();
.br

.EE
.RE
.PP
For simplest usage, some aliases are designed in the \fIMarkdownExtended\fP kernel:
.RS

.EX
// to parse a string content:
.br
\\MarkdownExtended\\MarkdownExtended::transformString($source [, $parser_options]);
.br

.br
// to parse a file content:
.br
\\MarkdownExtended\\MarkdownExtended::transformSource($filename [, $parser_options]);
.br

.EE
.RE
.PP
These two methods returns a \fIContent\fP object. To finally get an HTML
version, write:
.RS

.EX
\\MarkdownExtended\\MarkdownExtended::transformString($source [, $parser_options]);
.br
echo \\MarkdownExtended\\MarkdownExtended::getFullContent();
.br

.EE
.RE
.SH SEE ALSO
.PP
php(1), pcre(3), markdown-extended(3)

